require 'optparse'
require_relative 'smt_ar_processor'
 
 
 if __FILE__ == $0 then
	# Variables
	options = {} 
	model_directory = nil
	proj_classes = nil
	proj_relations = nil
	start_directory = Dir.getwd 
	property = nil
		     
		     
	# Set up OptionParser
	optparser = OptionParser.new do|opts|
		opts.banner = "Usage: ruby #{$0} [options] path\\to\\models\\folder" +
			"\n  This program translates a Ruby on Rails data model" +
			"\n  to a formal SMT-LIB specification." 
		 
		# Define the options								
		options[:outfile] = nil
		opts.on( '-o', '--smtspecoutput FILE', 'The name of the file to write SMT specification to' ) do |file|
		     options[:outfile] = file
		end
	     
		options[:property] = nil
		opts.on( '-p', '--property STRING',
		  'The property as a 3 (or 4) part string: propertyName class1 class2 (class3)' ) do |prop|
			options[:property] = prop
		end
		
		options[:projection_on] = nil
		opts.on( '-j', '--projection_on', 'Perform projection on the property specified' ) do
			options[:projection_on] = true
		end	

		options[:projection_info] = nil
		opts.on( '-f', '--projectionfile FILE', '(When property not specified) The name of the file that lists the Rails classes and SMT relations to project onto (one per line)' ) do |name|
			options[:projection_info] = name
		end	

		options[:delclass] = nil
		opts.on( '-d', '--deleteclass CLASSNAME', 'Name of the class to generate a delete property stub for' ) do |name|
			options[:delclass] = name
		end	
		
		# This displays the help screen
		opts.on( '-h', 'Display this screen' ) do
			puts opts
			exit
		end
	end
	
	
	# Parse options off of command line
	optparser.parse!
	
	 # Get models folder
	if ARGV.length == 1  #model directory should be the only thing left on the command line
		model_directory = ARGV[0]
	else
		puts "  Internal error: Translator invoked incorrectly."
		exit
	end
		     
	# Get property, if specified
	property = options[:property].split  if options[:property]
	
	# Get contents of projection info file, if any
	projfile = options[:projection_info]
	if projfile
		if not File.file? projfile
			puts "Projection file not found (#{projfile}).  Projection will not be performed."
		else
			proj_classes = []
			proj_relations = []
			file = File.open(projfile, 'r')
			
			file.each_line do|line|
				line = line.strip
				if line[0] =~ /[A-Z]/  # If entry starts with capital letter, assume it's a class
					proj_classes.push line
				else # otherwise it's a relation
					proj_relations.push line
				end
			end
			file.close
		end
	end
	
	 # Process each model file in the models folder
	 begin
		 Dir.chdir(model_directory)
	 rescue SystemCallError => e
		print "Error navigating to directory: " + e.to_s + "\n"
		exit
	 end
	 
	 Dir.foreach(".") do |filename| 
		if filename =~ /\.rb$/ and File.file?(filename) then
			# Get contents of model files
			file = File.open(filename, 'r')
			file_contents = ""
			file.each {|line| file_contents << line }
			file.close

			# Parse
			s_exp = RubyParser.new.parse file_contents			
			#print s_exp  # print the s-expression generated
			ActiveRecordProcessor.new.process(s_exp)
		end
	end
	Dir.chdir start_directory
	
	# Translate to SMT-LIB and Print specification
	ActiveRecordProcessor.translate_and_print_smtspec( options[:outfile], proj_classes,
		proj_relations, options[:delclass], property, options[:projection_on] )

end
