#!/usr/bin/ruby -w


if __FILE__ == $0 then
	 # Check command arguments
	 if ARGV.length != 1 then
		 print "  USAGE: ruby ", $0, " [filename] \n" 
		 print "   [filename] The path and name of the SMT-LIB specification\n"
		 print "   This script prints various statistics for an SMT-LIB specification.\n"
		 exit
	 end


	filename = ARGV[0]
	z3script = ""
	
	#Get file contents
	begin
		file = File.open(filename, 'r')
		file.each {|line| z3script << line }
		file.close
		
	# in case of any errors...
	rescue => err
		puts "Exception: #{err}"
		err
	end
	
	#Statistic gathering
	num_sorts = 0
	num_funs = 0
	num_quantvars = 0
	num_asserts = 0
	num_quants = 0
	num_opers = 0
	
	num_sorts = z3script.scan('declare-sort').count
	num_funs = z3script.scan('declare-fun').count
	num_quantvars = z3script.scan(/\([a-z] /).count + z3script.scan(/\([a-z][0-9] /).count
	num_asserts = z3script.scan('(assert').count
	num_quants = z3script.scan('(forall').count + z3script.scan('(exists').count
	num_opers = z3script.scan('(=').count +
				z3script.scan('(not').count +
				z3script.scan('(ite').count +
				z3script.scan('(and').count +
				z3script.scan('(or').count +
				z3script.scan('(iff').count +
				z3script.scan('(implies').count +
				z3script.scan(/ [a-z]\)/).count +
				z3script.scan(/ [a-z][0-9]\)/).count
	
	
	#Statistic printing
	print "\n# sorts: ", num_sorts
	print "\n# functions: ", num_funs
	print "\n# quantified variables: ", num_quantvars
	print "\n-"
	print "\n# asserts: ", num_asserts
	print "\n# quantifiers: ", num_quants
	print "\n# operators: ", num_opers
	
	print "\n\nTOTAL variables: ", num_sorts + num_funs + num_quantvars
	print "\nTOTAL clauses: ", num_asserts + num_quants + num_opers
	print "\n\n"
	
end

